﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinEditors;
using Infragistics.Win.UltraWinGrid;
using System.Globalization;

namespace CurrencyConverter
{
    #region frmMain class
    public partial class frmMain : Form
    {
        private DataTable table = null;

        public frmMain()
        {
            InitializeComponent();

            this.grid.Dock = DockStyle.Fill;
            this.grid.InitializeLayout += new InitializeLayoutEventHandler(this.grid_InitializeLayout);
            this.grid.InitializeRow += new InitializeRowEventHandler(this.grid_InitializeRow);
            this.grid.DataSource = this.Data;

        }

        private void grid_InitializeRow(object sender, InitializeRowEventArgs e)
        {
            //  Assign an EditorWithText, using a custom EmbeddableEditorOwnerBase
            //  implementation, to each cell in the 'Currency' column. This will 
            //  present the cell values to the user as relative to a specific culture's
            //  currency system, while preserving the underlying cell values as relative
            //  to the US dollar.
            UltraGridRow row = e.Row;
            string cultureID = row.Cells["CultureID"].Text;
            CultureInfo culture = CultureInfo.GetCultureInfo(cultureID);
            decimal conversionRate = (decimal)(row.Cells["ConversionRate"].Value);
            EditorWithText editor = new EditorWithText( new CurrencyFormatEditorOwner(culture) );
            editor.DataFilter = new CurrencyConversionDataFilter( conversionRate, culture.NumberFormat.CurrencyDecimalDigits );

            UltraGridCell cell = row.Cells["Currency"];
            cell.Editor = editor;
        }

        private void grid_InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            ColumnsCollection columns = e.Layout.Bands[0].Columns;
            foreach ( UltraGridColumn column in columns )
            {
                string key = column.Key;

                switch ( key )
                {
                    //  Hide the columns we don't want to user to see.
                    case "ConversionRate":
                    case "CultureID":
                        column.Hidden = true;
                        break;

                }
            }
        }

        private DataTable Data
        {
            get
            {
                if ( this.table == null )
                {
                    this.table = new DataTable();

                    //  Add columns
                    this.table.Columns.Add( "CultureName", typeof(string) );
                    this.table.Columns.Add( "CultureID", typeof(string) );
                    this.table.Columns.Add( "ConversionRate", typeof(decimal) );
                    this.table.Columns.Add( "Currency", typeof(decimal) );

                    //  Add rows
                    this.table.Rows.Add( new object[]{ "United States", "en-US", 1m, 100m } );
                    this.table.Rows.Add( new object[]{ "Great Britain", "en-GB", .665208m, 100m } );
                    this.table.Rows.Add( new object[]{ "Japan", "ja-JP", 88.5698m, 100m } );
                    this.table.Rows.Add( new object[]{ "Russia", "ru-RU", 30.8356m, 100m } );
                }

                return this.table;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            //  Display the active cell's value in a TextBox to show
            //  that while the cell displays it in units of a specific
            //  culture's currency, the underlying value is preserved
            //  in units of US dollars.
            this.textBox1.Text =
                this.grid.ActiveCell != null && this.grid.ActiveCell.Value != null ?
                this.grid.ActiveCell.Value.ToString() :
                string.Empty;
        }
    }
    #endregion frmMain class

    #region CurrencyFormatEditorOwner class
    /// <summary>
    /// EmbeddableEditorOwnerBase-derived class which provides currency formatting
    /// information for UltraGridCells.
    /// </summary>
    public class CurrencyFormatEditorOwner : DefaultEditorOwner
    {
        IFormatProvider formatProvider = null;

        public CurrencyFormatEditorOwner( IFormatProvider formatProvider ) { this.formatProvider = formatProvider; }

        /// <summary>
        /// Provides formatting information.
        /// </summary>
        public override void GetFormatInfo(object ownerContext, out string format, out IFormatProvider provider)
        {
            provider = this.formatProvider;
            format = "c";
        }
    }
    #endregion CurrencyFormatEditorOwner class

    #region CurrencyConversionDataFilter class
    /// <summary>
    /// IEditorDataFilter implementation which performs bi-directional
    /// currency rate conversions.
    /// </summary>
    public class CurrencyConversionDataFilter : IEditorDataFilter
    {
        #region Member variables
        private decimal conversionRate = 1m;
        private int currencyDecimalDigits = CultureInfo.InvariantCulture.NumberFormat.CurrencyDecimalDigits;
        #endregion Member variables

        #region Constructors

        /// <summary>
        /// Creates a new instance.
        /// </summary>
        /// <param name="conversionRateColumnKey">The key of the column that contains the rate of conversion.</param>
        public CurrencyConversionDataFilter( decimal conversionRate ) : this( conversionRate, 0 )
        {
        }

        /// <summary>
        /// Creates a new instance.
        /// </summary>
        /// <param name="conversionRateColumnKey">The key of the column that contains the rate of conversion.</param>
        /// <param name="currencyDecimalDigits">The number of decimal digits to which the value is rounded.</param>
        public CurrencyConversionDataFilter( decimal conversionRate, int currencyDecimalDigits )
        {
            this.conversionRate = conversionRate;

            if ( currencyDecimalDigits > 0 )
                this.currencyDecimalDigits = currencyDecimalDigits;
        }

        #endregion Constructors

        #region IEditorDataFilter Members

        object IEditorDataFilter.Convert(EditorDataFilterConvertArgs conversionArgs)
        {
            ConversionDirection direction = conversionArgs.Direction;
            UltraGridCell cell = conversionArgs.Context as UltraGridCell;
            UltraGridRow row = cell != null ? cell.Row : null;

            if ( cell == null || row == null || this.conversionRate == 1m )
                return conversionArgs.Value;

            //  Get the decimal value
            object rawValue = conversionArgs.Value;
            decimal value = 0m;

            if ( rawValue is string )
                decimal.TryParse(rawValue as string, out value);
            else
            if ( rawValue is decimal )
                value = (decimal)rawValue;

            if ( value == 0m )
                return conversionArgs.Value;

            switch ( direction )
            {
                //  For the 'EditorToOwner' conversion, i.e., the conversion
                //  that takes place when a cell exits edit mode, divide the
                //  value by the conversion rate.
                case ConversionDirection.EditorToOwner:
                {                    
                    conversionArgs.Handled = true;
                    return Math.Round( (value / this.conversionRate), this.currencyDecimalDigits );
                }

                //  For the 'OwnerToEditor' conversion, i.e., the conversion
                //  that we do to show the converted value in the cell without
                //  changing the underlying value, multiply by the conversion
                //  rate.
                case ConversionDirection.OwnerToEditor:
                {                    
                    conversionArgs.Handled = true;
                    return value * this.conversionRate;
                }
            }

            return conversionArgs.Value;
        }

        #endregion
    }
    #endregion CurrencyConversionDataFilter class
}
